#!/usr/bin/php
<?php
//
// pvoutput v. 0.3 - Dec 28, 2017
// This script will send data to pvoutput.com
//
// How to:
// 1) Copy the script in /var/www/comapps
//    change owner with command 'chown www-data.www-data pvoutput.php'
//    change permission with command 'chmod +x pvoutput.php'
//    make a link with command 'ln -s /var/www/comapps/pvoutput.php /usr/bin/pvoutput'
// 2) Edit this file and set
//    PVOUTPUT SYSID with your SYSID
//    PVOUTPUT KEY with your APIKEY
//    VoltageID will be set to grid voltage indicator 
//       or with "" for no voltage indicator
//    For temperature value you have to select feed by setting $TempFeed variable
//       TempFeed:
//             ""   for no Temperature value
//             "M"  for meterN indicator 
//             "W"  for Wunderground 
//       If selecting M for meterN feed need to be set $TempID variable with 
//       meterN Temperature indicator otherwise set $TempID = "";
//       If selecting W for Wunderground feed need to be set $wkey and $wstationID variables.
//       otherwise set $wkey = "" and $wstationID = "";
// 3) in this file adjust pathtomn if necessary
// 4) In meterN, set a new indicator whit this parameters:
//    Name: 'PVOutput'
//    ID: 'PVOutput'
//    Mode: 'State'
//    Command: pvoutput
//    Unit: X
//
//
// Copyright (C) 2017 by Garluk
//

/*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// PVOUTPUT SYSID
$SYSID='yourSYSID';

// PVOUTPUT KEY
$APIKEY='yuorAPIKEY';

// Voltage indicator ("" for no voltage indicator)
$VoltageID = "Tensione1";

// Temperature
// For temperature value you have to select feed by setting $TempFeed variable
// TempFeed:
//             M  for meterN indicator 
//             W  for Wunderground 
// If selecting M for meterN feed need to be set $TempID variable with 
// meterN Temperature indicator otherwise set $TempID = "";
// If selecting W for Wunderground feed need to be set $wkey and $wstationID variables.
$TempFeed = "M";

// Temperature indicator ("" for no temperature indicator)
$TempID = "";

// Wunderground kwy ("" for no Wunderground key)
$wkey = "yourWundergroundkey";
// Wunderground stationID ("" for no Wunderground stationID)
$wstationID = "yourstationID";

// Path to metern (edit if necessary)
$pathtomn = '/var/www/metern';


//
// No edit should be needed bellow
//
// -----------------------------------------------------------------------------

if (isset($_SERVER['REMOTE_ADDR'])) {
    die("Direct access not permitted\n");
}

// Test if $TempFeed is valid
if (($TempFeed != "M") && ($TempFeed != "W")) {
	die("TempFeed is not valid\n");
}

define('checkaccess', true);
include("$pathtomn/config/config_main.php");
include("$pathtomn/config/memory.php");

// open mN shm memory
$data = file_get_contents($MEMORY);
$memarray = json_decode($data, true);

// open mN shm live memory
$data = file_get_contents($ILIVEMEMORY);
$ilivemem = json_decode($data, true);

// open my shm memory
if (file_exists($TMPFS . '/myPVO_MEMORY.json')) {
	$data1 = file_get_contents($TMPFS . '/myPVO_MEMORY.json');
	$data = $data1;
	$mymemarray = json_decode($data1, true);
} else {
	$mymemarray = array();
}

// Retrieve temperature
$sendT = "no";
switch ($TempFeed) {
	case 'M':
		if (array_key_exists("$TempID", $ilivemem)) {
			$Temp =  $ilivemem["$TempID"];
			$sendT = "ok";
		}
		break;
	case 'W':
		$json = null;
		$ctx = stream_context_create(array('http'=>
			array(
				'timeout' => 3
			)
		));
		$json = file_get_contents("http://api.wunderground.com/api/$wkey/conditions/q/pws:$wstationID.json", false, $ctx);
		$weatherData = json_decode($json,true);
		if (isset($weatherData['current_observation']['temp_c'])) {
			$Temp = round($weatherData['current_observation']['temp_c'], 1);
			$sendT = "ok";
		}
		break;
}

date_default_timezone_set($DTZ);
$today = date('Ymd');
$time = date('H:i', mktime(date('H'), date('i') - 1, 0, date('m'), date('d'), date('Y')));

if (!isset($mymemarray["5minflag"])) {
	// Retrieve initial value
	if (file_exists("$pathtomn/data/csv/$today.csv")) {
		$lines = file("$pathtomn/data/csv/$today.csv");
		$first_array = preg_split('/,/', $lines[1]);
		$contalines = count($lines);
		if ($contalines > 2) { // daily file
			$last_array = preg_split('/,/', $lines[$contalines - 1]);
			$prev_array = preg_split('/,/', $lines[$contalines - 2]);
		} else {
			$last_array = $first_array;
			$prev_array = $first_array;
		}
		
		for ($i = 1; $i <= $NUMMETER; $i++) {
			$mymemarray["Startvalue$i"] = trim($first_array[$i]);
		}
		
		$hour  = substr($last_array[0], 0, 2);
		$minute = substr($last_array[0], 3, 2);
		$prevhour = substr($prev_array[0], 0, 2);
		$prevminute = substr($prev_array[0], 3, 2);
		$UTCdate = strtotime("$today" . $hour . ':' . $minute);
		$diffUTCdate = strtotime("$today" . $prevhour . ':' . $prevminute);
		$diffTime = $UTCdate - $diffUTCdate;

		if ($diffTime >= 300) {
			for ($i = 1; $i <= $NUMMETER; $i++) {
				$mymemarray["Prevvalue$i"] = trim($prev_array[$i]);
			}
		} else {
			for ($i = 1; $i <= $NUMMETER; $i++) {
				$mymemarray["Prevvalue$i"] = 0;
			}
		}
		$mymemarray["PrevUTCdate"] = $UTCdate;
	} else {
		// Sstarting from scratch
		for ($i = 1; $i <= $NUMMETER; $i++) {
			if (${'TYPE' . $i} != 'Sensor') {
				$mymemarray["Startvalue$i"] = 0;
				$mymemarray["Prevvalue$i"] = 0;
			}
		}
		$mymemarray["PrevUTCdate"] = strtotime("$today " . $time);
	}
	
	$mymemarray["5minflag"] = false;
	$mymemarray["Prevdate"] = $today;
} else {
	// Init variables
	$prodStart = 0.0;
	$prodPrev = 0.0;
	$prodNow = 0.0;
	$prod = 0.0;
	$consStart = 0.0;
	$consPrev = 0.0;
	$consNow = 0.0;
	$cons = 0.0;
	$prodlist = array();
	$conslist = array();

	// Read MeterN meters types and config
	for ($i = 1; $i <= $NUMMETER; $i++) {
		include("$pathtomn/config/config_met$i.php");
		if (${'PROD' . $i} == 1) {
			$prodlist[] = $i;
		}
		if (${'PROD' . $i} == 2) {
			$conslist[] = $i;
		}
	}

	for ($i = 0; $i < count($prodlist); $i++) {
		$prodStart += $mymemarray["Startvalue$prodlist[$i]"];
		$prodPrev += $mymemarray["Prevvalue$prodlist[$i]"];
		$prodNow += $memarray["Totalcounter$prodlist[$i]"];
		if (($memarray["Totalcounter$prodlist[$i]"] < $mymemarray["Prevvalue$prodlist[$i]"]) && (${'PASSO' . $prodlist[$i]} > 0)) { // counter pass over
			$prodPrev += ${'PASSO' . $prodlist[$i]};
		}
		if (($memarray["Totalcounter$prodlist[$i]"] < $mymemarray["Startvalue$prodlist[$i]"]) && (${'PASSO' . $prodlist[$i]} > 0)) { // counter pass over
			$prodNow += ${'PASSO' . $prodlist[$i]};
		}
	}
	$prod = $prodNow - $prodStart; // Daily energy Production
	if ($prod < 0) {
		$prod = 0;
	}
	
	for ($i = 0; $i < count($conslist); $i++) {
		$consStart += $mymemarray["Startvalue$conslist[$i]"];
		$consPrev += $mymemarray["Prevvalue$conslist[$i]"];
		$consNow += $memarray["Totalcounter$conslist[$i]"];
		if (($memarray["Totalcounter$conslist[$i]"] < $mymemarray["Prevvalue$conslist[$i]"]) && ($conslist[$i] > 0)) { // counter pass over
			$consPrev += ${'PASSO' . $conslist[$i]};
		}
		if (($memarray["Totalcounter$conslist[$i]"] < $mymemarray["Startvalue$conslist[$i]"]) && ($conslist[$i] > 0)) { // counter pass over
			$consNow += ${'PASSO' . $conslist[$i]};
		}
	}
	$cons = $consNow - $consStart; // Daily energy Consumption
	
	$diffTime = strtotime("$today " . $time) - $mymemarray["PrevUTCdate"];
	if ($diffTime >= 300) {
		$prodAvgP = round((($prodNow - $prodPrev) * 3600) / $diffTime, 0); // Avg Power Production
		$consAvgP = round((($consNow - $consPrev) * 3600) / $diffTime, 0); // Avg Power Consumption
	} else {
		$prodAvgP = 0;
		$consAvgP = 0;
	}
	
	$params = 'd=' . $today . '&t=' . $time;
	$params .= '&v1=' . $prod . '&v2=' . $prodAvgP;
	$params .= '&v3=' . $cons . '&v4=' . $consAvgP;
	if ($sendT == "ok") {
		$params .=  '&v5=' . $Temp;
	}
	if (array_key_exists("$VoltageID", $ilivemem)) {
		$Volt =  $ilivemem["$VoltageID"];
		$params .=  '&v6=' . $Volt;
	} else {
		$Volt = 0;
	}

	if ($mymemarray["5minflag"] && !$memarray["5minflag"]) {
		// Update prev data
		for ($i = 1; $i <= $NUMMETER; $i++) {
			if (${'TYPE' . $i} != 'Sensor') {
				$mymemarray["Prevvalue$i"] = $memarray["Totalcounter$i"];
				if ($mymemarray["Prevdate"] != $today) {
					$mymemarray["Startvalue$i"] = $memarray["Totalcounter$i"];
				}
			}
		}
		$mymemarray["PrevUTCdate"] = strtotime("$today " . $time);
		$mymemarray["Prevdate"] = $today;
		// Send data
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, 'http://pvoutput.org/service/r2/addstatus.jsp');
		curl_setopt($ch, CURLOPT_TIMEOUT, 10);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
			'X-Pvoutput-Apikey: ' . $APIKEY,
			'X-Pvoutput-SystemId: ' . $SYSID
		));
		curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
		$pvo = curl_exec($ch);
		curl_close($ch);
		if ($DEBUG) {
			$stringData .= "\nSend for SYSID $SYSID : $today $time";
			$stringData .= "\nProduction: daily energy $prod Wh  - Avg Power $prodAvgP W";
			$stringData .= "\nConsumption : daily energy $cons Wh Avg. power $consAvgP W\n";
			$stringData .= "\nTemperature $Temp °C";
			$stringData .= "\nGrid voltage $Volt V";
			$stringData .= "\nPVoutput returned: $pvo";
			$myFile = "$pathtomn/data/metern.err";
//			$myFile = "/dev/shm/metern.log";
			file_put_contents($myFile, $stringData, FILE_APPEND);
		}
	}

	$mymemarray["5minflag"] = $memarray["5minflag"];
}

$data1 = json_encode($mymemarray);
if ($data != $data1) { // Reduce write
	$data1 = json_encode($mymemarray);
	file_put_contents($TMPFS . '/myPVO_MEMORY.json', $data1);
}

if (!isset($pvo)) {
	$ok = 1;
} else {
	if ($pvo === false) {
		$ok = 0;
	} else if (substr($pvo, 0, 2) == "OK") {
		$ok = 1;
	} else {
		$ok = 0;
	}
}
echo("PVOutput($ok*X)");
?>
